import xfox

@xfox.addfunc(xfox.funcs, name="roleGrant")
async def etcx2(user_id, *args, **kwargs):
    ctx = (kwargs.get("ctx_command") or kwargs.get("ctx_message_env") or kwargs.get("ctx_slash_env") or 
           kwargs.get("ctx_reaction_env") or kwargs.get("ctx_reaction_remove_env") or kwargs.get("ctx_interaction_env") or 
           (kwargs.get("ctx_message_edit_env", (None,))[0]) or kwargs.get("ctx_message_delete_env") or kwargs.get("ctx_bulk_message_delete_env"))

    if not ctx:
        print("[DEBUG ROLEGRANT] No se encontró un contexto válido, Contacta con Soporte")
        raise ValueError(":x: No se encontró un contexto válido.")

    miembro = ctx.guild.get_member(int(user_id))  # Obtener usuario por ID
    if not miembro:
       print("[DEBUG ROLEGRANT] No se encontró al usuario con ID `{user_id}`.")
       raise ValueError(f":x: No se encontró al usuario con ID `{user_id}`.")

    # Verificar si el bot tiene permiso de administrar roles
    if not ctx.guild.me.guild_permissions.manage_roles:
        print("[DEBUG ROLEGRANT] El bot no tiene permisos para administrar roles.")
        raise ValueError(f":x: No tengo permisos para administrar roles.")

    roles_a_agregar = []
    roles_a_remover = []
    bot_highest_role = ctx.guild.me.top_role  # Rol más alto del bot

    for arg in args:
        if arg.startswith('+'):
            role_id = int(arg[1:])  # Obtener ID quitando el "+"
            rol = ctx.guild.get_role(role_id)
            if rol:
                if rol.position >= bot_highest_role.position:
                    print("[DEBUG ROLEGRANT] No puedo agregar el rol `{rol.name}` porque es igual o superior a mi rol más alto.")
                    raise ValueError(f":x: No puedo agregar el rol `{rol.name}` porque es igual o superior a mi rol más alto.")
                roles_a_agregar.append(rol)
            else:
                print(f"[DEBUG ROLEGRANT] No se encontró el rol con ID `{role_id}`, $roleGrant[..;+{role_id}]")
                raise ValueError(f":x: No se encontró el rol con ID `{role_id}`, `$roleGrant[..;+{role_id}]`.")
            

        elif arg.startswith('-'):
            role_id = int(arg[1:])  # Obtener ID quitando el "-"
            rol = ctx.guild.get_role(role_id)
            if rol:
                if rol.position >= bot_highest_role.position:
                    print("[DEBUG ROLEGRANT] No puedo quitar el rol `{rol.name}` porque es igual o superior a mi rol más alto.")
                    raise ValueError(f":x: No puedo quitar el rol `{rol.name}` porque es igual o superior a mi rol más alto.")
                roles_a_remover.append(rol)
            else:
                print(f"[DEBUG ROLEGRANT] No se encontró el rol con ID `{role_id}, `$roleGrant[..;-{role_id}]`")
                raise  ValueError(f":x: No se encontró el rol con ID `{role_id}`, `$roleGrant[..;-{role_id}]`.")

    # Aplicar los cambios de roles
    if roles_a_agregar:
        await miembro.add_roles(*roles_a_agregar)
    if roles_a_remover:
        await miembro.remove_roles(*roles_a_remover)
    
    return ""



import xfox
import discord

@xfox.addfunc(xfox.funcs)
async def channelSystem(*args, **kwargs):
    if "ctx_join_member_env" in kwargs:
        n = kwargs["ctx_join_member_env"].guild.system_channel
        return n
    return ""



import xfox
import discord

@xfox.addfunc(xfox.funcs)
async def categorysNames(separator:str, guild_id=None, *args, **kwargs):
    # Obtenemos los diferentes contextos desde kwargs
    ctx_command = kwargs.get("ctx_command")
    ctx_slash_env = kwargs.get("ctx_slash_env")
    ctx_message_env = kwargs.get("ctx_message_env")
    ctx_reaction_env = kwargs.get("ctx_reaction_env")
    ctx_reaction_remove_env = kwargs.get("ctx_reaction_remove_env")
    ctx_interaction_env = kwargs.get("ctx_interaction_env")
    ctx_message_edit_env = (kwargs.get("ctx_message_edit_env", (None,))[0]) 
    ctx_message_delete_env = kwargs.get("ctx_message_delete_env")
    ctx_join_member_env = kwargs.get("ctx_join_member_env")
    ctx_leave_member_env = kwargs.get("ctx_leave_member_env")

    # Validar el separador para evitar problemas
    separator = separator.strip()  # Eliminamos espacios en blanco antes y después


    # Si se pasa un guild_id, intentamos obtener el servidor por ID
    if guild_id:
        guild = discord.utils.get(ctx_command.bot.guilds, id=int(guild_id))
    else:
        # Si no se pasa guild_id, intentamos obtener el servidor del contexto
        guild = ctx_command.guild if ctx_command else None

        # Si no conseguimos el guild en ctx_command, intentamos en otros contextos
        if not guild:
            if ctx_slash_env:
                guild = ctx_slash_env.guild
            elif ctx_message_env:
                guild = ctx_message_env.guild
            elif ctx_interaction_env:
                guild = ctx_interaction_env.guild
            elif ctx_message_edit_env:
                guild = ctx_message_edit_env.guild
            elif ctx_message_delete_env:
                guild = ctx_message_delete_env.guild
            elif ctx_join_member_env:
                guild = ctx_join_member_env.guild
            elif ctx_leave_member_env:
                guild = ctx_leave_member_env.guild
            elif ctx_reaction_env:
                guild = ctx_reaction_env.guild  # Usamos ctx_reaction_env para obtener el guild
            elif ctx_reaction_remove_env:
                guild = ctx_reaction_remove_env.guild  # Usamos ctx_reaction_remove_env para obtener el guild

    # Si encontramos el servidor, devolvemos los nombres de las categorías como texto
    if guild:
        # Filtramos las categorías (solo los canales de tipo 'CategoryChannel')
        category_names = [category.name for category in guild.categories]

        # Si encontramos categorías, las unimos en un string usando el separator
        if category_names:
            return separator.join(category_names)
        else:
            print("[DEBUG CATEGORYSNAMES] No se encontraron categorías en este servidor.")
            return ""
    else:
        print("[DEBUG CATEGORYSNAMES] No se pudo obtener el servidor.")
        return ""



import xfox
import discord

@xfox.addfunc(xfox.funcs, name="clear")
async def clear_func(amount: int, user_id: str = None, remove_pinned: bool = False, *args, **kwargs):
    ctx = (kwargs.get("ctx_command") or kwargs.get("ctx_message_env") or kwargs.get("ctx_slash_env") or 
           kwargs.get("ctx_reaction_env") or kwargs.get("ctx_reaction_remove_env") or kwargs.get("ctx_interaction_env") or 
           (kwargs.get("ctx_message_edit_env", (None,))[0]) or kwargs.get("ctx_message_delete_env") or kwargs.get("ctx_bulk_message_delete_env"))
    
    if not ctx or not hasattr(ctx, 'channel'):
        return
    
    def check(msg):
        if user_id and str(msg.author.id) != user_id:
            return False
        if not remove_pinned and msg.pinned:
            return False
        return True
    
    await ctx.channel.purge(limit=amount, check=check)
    return ""
